Package: bayesplot
Type: Package
Title: Plotting for Bayesian Models
Version: 1.15.0
Date: 2025-12-11
Authors@R: c(person("Jonah", "Gabry", role = c("aut", "cre"), email = "jgabry@gmail.com"),
             person("Tristan", "Mahr", role = "aut", comment = c(ORCID = "0000-0002-8890-5116")),
             person("Paul-Christian", "Bürkner", role = "ctb"),
             person("Martin", "Modrák", role = "ctb"),
             person("Malcolm", "Barrett", role = "ctb"),
             person("Frank", "Weber", role = "ctb"),
             person("Eduardo", "Coronado Sroka", role = "ctb"),
             person("Teemu", "Sailynoja", role = "ctb"),
             person("Aki", "Vehtari", role = "ctb"), 
             person("Behram", "Ulukır", role = "ctb"), 
             person("Visruth", "Srimath Kandali", role = "ctb"))
Maintainer: Jonah Gabry <jgabry@gmail.com>
Description: Plotting functions for posterior analysis, MCMC diagnostics,
    prior and posterior predictive checks, and other visualizations
    to support the applied Bayesian workflow advocated in
    Gabry, Simpson, Vehtari, Betancourt, and Gelman (2019) <doi:10.1111/rssa.12378>.
    The package is designed not only to provide convenient functionality
    for users, but also a common set of functions that can be easily used by
    developers working on a variety of R packages for Bayesian modeling,
    particularly (but not exclusively) packages interfacing with 'Stan'.
License: GPL (>= 3)
URL: https://mc-stan.org/bayesplot/
BugReports: https://github.com/stan-dev/bayesplot/issues/
SystemRequirements: pandoc (>= 1.12.3), pandoc-citeproc
Depends: R (>= 4.1.0)
Imports: dplyr (>= 0.8.0), ggplot2 (>= 3.4.0), ggridges (>= 0.5.5),
        glue, posterior, reshape2, rlang (>= 0.3.0), stats, tibble (>=
        2.0.0), tidyr, tidyselect, utils
Suggests: ggdist, ggfortify, gridExtra (>= 2.2.1), hexbin, knitr (>=
        1.16), loo (>= 2.0.0), RColorBrewer, rmarkdown (>= 1.0.0),
        rstan (>= 2.17.1), rstanarm (>= 2.17.4), rstantools (>= 1.5.0),
        scales, shinystan (>= 2.3.0), survival, testthat (>= 3.0.0),
        vdiffr (>= 1.0.2)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-12-11 17:39:24 UTC; jgabry
Author: Jonah Gabry [aut, cre],
  Tristan Mahr [aut] (ORCID: <https://orcid.org/0000-0002-8890-5116>),
  Paul-Christian Bürkner [ctb],
  Martin Modrák [ctb],
  Malcolm Barrett [ctb],
  Frank Weber [ctb],
  Eduardo Coronado Sroka [ctb],
  Teemu Sailynoja [ctb],
  Aki Vehtari [ctb],
  Behram Ulukır [ctb],
  Visruth Srimath Kandali [ctb]
Repository: CRAN
Date/Publication: 2025-12-12 07:10:55 UTC
Built: R 4.6.0; ; 2026-04-29 10:22:40 UTC; unix
