# Generated by roxygen2: do not edit by hand

S3method("[",neff_ratio)
S3method("[",rhat)
S3method(apply_transformations,array)
S3method(apply_transformations,matrix)
S3method(diagnostic_factor,neff_ratio)
S3method(diagnostic_factor,rhat)
S3method(log_posterior,CmdStanMCMC)
S3method(log_posterior,stanfit)
S3method(log_posterior,stanreg)
S3method(melt_mcmc,matrix)
S3method(melt_mcmc,mcmc_array)
S3method(neff_ratio,CmdStanMCMC)
S3method(neff_ratio,stanfit)
S3method(neff_ratio,stanreg)
S3method(num_chains,data.frame)
S3method(num_chains,mcmc_array)
S3method(num_iters,data.frame)
S3method(num_iters,mcmc_array)
S3method(num_params,data.frame)
S3method(num_params,mcmc_array)
S3method(nuts_params,CmdStanMCMC)
S3method(nuts_params,list)
S3method(nuts_params,stanfit)
S3method(nuts_params,stanreg)
S3method(parameter_names,array)
S3method(parameter_names,default)
S3method(parameter_names,matrix)
S3method(plot,bayesplot_grid)
S3method(plot,bayesplot_scheme)
S3method(pp_check,default)
S3method(print,bayesplot_function_list)
S3method(print,bayesplot_grid)
S3method(print,bayesplot_scheme)
S3method(rhat,CmdStanMCMC)
S3method(rhat,stanfit)
S3method(rhat,stanreg)
export(abline_01)
export(available_mcmc)
export(available_ppc)
export(available_ppd)
export(bayesplot_grid)
export(bayesplot_theme_get)
export(bayesplot_theme_replace)
export(bayesplot_theme_set)
export(bayesplot_theme_update)
export(color_scheme_get)
export(color_scheme_set)
export(color_scheme_view)
export(example_group_data)
export(example_mcmc_draws)
export(example_x_data)
export(example_y_data)
export(example_yrep_draws)
export(facet_bg)
export(facet_text)
export(grid_lines)
export(hline_0)
export(hline_at)
export(lbub)
export(legend_move)
export(legend_none)
export(legend_text)
export(log_posterior)
export(mcmc_acf)
export(mcmc_acf_bar)
export(mcmc_areas)
export(mcmc_areas_data)
export(mcmc_areas_ridges)
export(mcmc_areas_ridges_data)
export(mcmc_combo)
export(mcmc_dens)
export(mcmc_dens_chains)
export(mcmc_dens_chains_data)
export(mcmc_dens_overlay)
export(mcmc_hex)
export(mcmc_hist)
export(mcmc_hist_by_chain)
export(mcmc_intervals)
export(mcmc_intervals_data)
export(mcmc_neff)
export(mcmc_neff_data)
export(mcmc_neff_hist)
export(mcmc_nuts_acceptance)
export(mcmc_nuts_divergence)
export(mcmc_nuts_energy)
export(mcmc_nuts_stepsize)
export(mcmc_nuts_treedepth)
export(mcmc_pairs)
export(mcmc_parcoord)
export(mcmc_parcoord_data)
export(mcmc_rank_ecdf)
export(mcmc_rank_hist)
export(mcmc_rank_overlay)
export(mcmc_recover_hist)
export(mcmc_recover_intervals)
export(mcmc_recover_scatter)
export(mcmc_rhat)
export(mcmc_rhat_data)
export(mcmc_rhat_hist)
export(mcmc_scatter)
export(mcmc_trace)
export(mcmc_trace_data)
export(mcmc_trace_highlight)
export(mcmc_violin)
export(neff_ratio)
export(nuts_params)
export(overlay_function)
export(pairs_condition)
export(pairs_style_np)
export(panel_bg)
export(param_glue)
export(param_range)
export(parcoord_style_np)
export(plot_bg)
export(pp_check)
export(ppc_bars)
export(ppc_bars_data)
export(ppc_bars_grouped)
export(ppc_boxplot)
export(ppc_data)
export(ppc_dens)
export(ppc_dens_overlay)
export(ppc_dens_overlay_grouped)
export(ppc_dots)
export(ppc_ecdf_overlay)
export(ppc_ecdf_overlay_grouped)
export(ppc_error_binned)
export(ppc_error_data)
export(ppc_error_hist)
export(ppc_error_hist_grouped)
export(ppc_error_scatter)
export(ppc_error_scatter_avg)
export(ppc_error_scatter_avg_grouped)
export(ppc_error_scatter_avg_vs_x)
export(ppc_freqpoly)
export(ppc_freqpoly_grouped)
export(ppc_hist)
export(ppc_intervals)
export(ppc_intervals_data)
export(ppc_intervals_grouped)
export(ppc_km_overlay)
export(ppc_km_overlay_grouped)
export(ppc_loo_intervals)
export(ppc_loo_pit)
export(ppc_loo_pit_data)
export(ppc_loo_pit_ecdf)
export(ppc_loo_pit_overlay)
export(ppc_loo_pit_qq)
export(ppc_loo_ribbon)
export(ppc_pit_ecdf)
export(ppc_pit_ecdf_grouped)
export(ppc_ribbon)
export(ppc_ribbon_data)
export(ppc_ribbon_grouped)
export(ppc_rootogram)
export(ppc_scatter)
export(ppc_scatter_avg)
export(ppc_scatter_avg_data)
export(ppc_scatter_avg_grouped)
export(ppc_scatter_data)
export(ppc_stat)
export(ppc_stat_2d)
export(ppc_stat_data)
export(ppc_stat_freqpoly)
export(ppc_stat_freqpoly_grouped)
export(ppc_stat_grouped)
export(ppc_violin_grouped)
export(ppd_boxplot)
export(ppd_data)
export(ppd_dens)
export(ppd_dens_overlay)
export(ppd_dots)
export(ppd_ecdf_overlay)
export(ppd_freqpoly)
export(ppd_freqpoly_grouped)
export(ppd_hist)
export(ppd_intervals)
export(ppd_intervals_data)
export(ppd_intervals_grouped)
export(ppd_ribbon)
export(ppd_ribbon_data)
export(ppd_ribbon_grouped)
export(ppd_stat)
export(ppd_stat_2d)
export(ppd_stat_data)
export(ppd_stat_freqpoly)
export(ppd_stat_freqpoly_grouped)
export(ppd_stat_grouped)
export(rhat)
export(scatter_style_np)
export(theme_default)
export(trace_style_np)
export(vars)
export(vline_0)
export(vline_at)
export(xaxis_text)
export(xaxis_ticks)
export(xaxis_title)
export(yaxis_text)
export(yaxis_ticks)
export(yaxis_title)
import(ggplot2)
import(rlang)
import(stats)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,one_of)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,top_n)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(ggplot2,"%+replace%")
